from ewokscore import task_discovery


def test_task_class_discovery():
    expected = [
        {
            "task_type": "class",
            "task_identifier": "ewokscore.tests.discover_module.MyTask1",
            "required_input_names": ["a"],
            "optional_input_names": ["b"],
            "output_names": ["result"],
            "category": "ewokscore",
        },
        {
            "task_type": "class",
            "task_identifier": "ewokscore.tests.discover_module.MyTask2",
            "required_input_names": ["a"],
            "optional_input_names": ["b"],
            "output_names": ["result"],
            "category": "ewokscore",
        },
    ]

    tasks = task_discovery.discover_tasks_from_modules()
    for task in expected:
        assert task not in tasks

    tasks = task_discovery.discover_tasks_from_modules(
        "ewokscore.tests.discover_module"
    )

    for task in expected:
        assert task in tasks
    assert len(tasks) == len(expected)

    tasks = task_discovery.discover_tasks_from_modules()
    for task in expected:
        assert task in tasks


def test_task_method_discovery():
    expected = [
        {
            "task_type": "method",
            "task_identifier": "ewokscore.tests.discover_module.run",
            "category": "ewokscore",
        },
        {
            "task_type": "method",
            "task_identifier": "ewokscore.tests.discover_module.myfunc",
            "category": "ewokscore",
        },
    ]
    tasks = task_discovery.discover_tasks_from_modules(
        "ewokscore.tests.discover_module", task_type="method"
    )
    for task in expected:
        assert task in tasks
    assert len(tasks) == len(expected)


def test_task_ppfmethod_discovery():
    expected = [
        {
            "task_type": "method",
            "task_identifier": "ewokscore.tests.discover_module.run",
            "category": "ewokscore",
        }
    ]
    tasks = task_discovery.discover_tasks_from_modules(
        "ewokscore.tests.discover_module", task_type="ppfmethod"
    )
    for task in expected:
        assert task in tasks
    assert len(tasks) == len(expected)



import sys
import inspect
from typing import Callable, Iterable, Optional

from ewoksutils.import_utils import qualname
from ewoksutils.import_utils import import_module

from .task import Task


def discover_tasks_from_modules(
    *module_names: Iterable[str], task_type="class"
) -> Iterable[dict]:
    return list(iter_discover_tasks_from_modules(*module_names, task_type=task_type))


def iter_discover_tasks_from_modules(
    *module_names: Iterable[str], task_type="class", reload: bool = False
) -> Iterable[dict]:
    if "" not in sys.path:
        # This happens when the python process was launched
        # through a python console script
        sys.path.append("")

    if task_type == "method":
        yield from _iter_method_tasks(*module_names, reload=reload)
    elif task_type == "ppfmethod":
        yield from _iter_method_tasks(
            *module_names, filter_method_name=lambda name: name == "run", reload=reload
        )
    elif task_type == "class":
        for module_name in module_names:
            import_module(module_name, reload=reload)
        yield from _iter_registered_tasks(*module_names)
    else:
        raise ValueError("Class type does not support discovery")


def _iter_registered_tasks(*filter_modules: Iterable[str]) -> Iterable[dict]:
    """Yields all task classes registered in the current process."""
    for cls in Task.get_subclasses():
        module = cls.__module__
        if filter_modules and not any(
            module.startswith(prefix) for prefix in filter_modules
        ):
            continue
        task_identifier = cls.class_registry_name()
        category = task_identifier.split(".")[0]
        yield {
            "task_type": "class",
            "task_identifier": task_identifier,
            "required_input_names": list(cls.required_input_names()),
            "optional_input_names": list(cls.optional_input_names()),
            "output_names": list(cls.output_names()),
            "category": category,
        }


def _iter_method_tasks(
    *module_names: Iterable[str],
    filter_method_name: Optional[Callable[[str], bool]] = None,
    reload: bool = False
) -> Iterable[dict]:
    """Yields all task methods from the provided module_names. The module_names will be will
    imported for discovery.
    """
    for module_name in module_names:
        mod = import_module(module_name, reload=reload)
        for method in inspect.getmembers(mod, inspect.isfunction):
            method_name, method_qn = method
            if filter_method_name and not filter_method_name(method_name):
                continue
            if method_name.startswith("_"):
                continue
            task_identifier = qualname(method_qn)
            category = task_identifier.split(".")[0]
            yield {
                "task_type": "method",
                "task_identifier": qualname(method_qn),
                "category": category,
            }



import os
import sys
from ewokscore.task import Task

WIN32 = sys.platform == "win32"


pyscript = r"""
if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument("--a", type=int, default=0)
    args = parser.parse_args()
    print("input a =", args.a)
    assert args.a == 10
"""


def test_python_script_task(tmpdir, varinfo, capsys):
    pyscriptname = tmpdir / "test.py"
    with open(pyscriptname, mode="w") as f:
        f.writelines(pyscript)

    task = Task.instantiate(
        "ScriptExecutorTask",
        inputs={"a": 10, "_script": str(pyscriptname)},
        varinfo=varinfo,
    )
    task.execute()
    assert task.done
    assert task.outputs.return_code == 0
    captured = capsys.readouterr()
    # assert captured.out == "10\n"
    assert captured.err == ""


if WIN32:
    shellscript = r"""@echo off

set a=0

:initial
if "%1"=="" goto done
echo              %1
set aux=%1
if "%aux:~0,1%"=="-" (
   set varname=%aux:~1,250%
) else (
   set "%varname%=%1"
   set varname=
)
shift
goto initial
:done

echo input a = %a%
if %a%==10 (
    echo exit 0
) else (
    echo exit 1
)
"""
else:
    shellscript = r"""a=0

while getopts u:a:f: flag
do
    case "${flag}" in
        a) a=${OPTARG};;
    esac
done

echo "input a = "$a
if [[ $a == "10" ]]; then
    exit 0
else
    exit 1
fi
"""


def test_shell_script_task(tmpdir, varinfo, capsys):
    if WIN32:
        ext = ".bat"
    else:
        ext = ".sh"
    filename = tmpdir / f"test{ext}"
    with open(filename, mode="w") as f:
        f.writelines(shellscript)
    if not WIN32:
        os.chmod(filename, 0o755)

    task = Task.instantiate(
        "ScriptExecutorTask",
        inputs={"a": 10, "_script": str(filename)},
        varinfo=varinfo,
    )
    task.execute()
    assert task.done
    assert task.outputs.return_code == 0
    captured = capsys.readouterr()
    # assert captured.out == "10\n"
    assert captured.err == ""
